using System.Reflection;
using FluentNHibernate.Cfg;
using FluentNHibernate.Cfg.Db;
using NHibernate;
using NHibernate.Cfg;
using NHibernate.Driver;
using NHibernate.Tool.hbm2ddl;
using Environment = NHibernate.Cfg.Environment;

namespace UnitTests.Utils
{
    public abstract class InMemoryDbTestFixtureBase
    {
        protected static ISessionFactory sessionFactory;
        private static Configuration _configuration;

        public static void InitialiseNHibernate(params Assembly[] assemblies)
        {
            if (sessionFactory != null)
                return;

            sessionFactory =  Fluently.Configure()
                                        .Database(
                                            SQLiteConfiguration.Standard
                                                .ProxyFactoryFactory(typeof(NHibernate.ByteCode.LinFu.ProxyFactoryFactory))
                                                .InMemory()
                                                .ShowSql
                                            )
                                        .Mappings((m) =>
                                                      {
                                                          foreach (var assembly in assemblies)
                                                              m.FluentMappings.AddFromAssembly(assembly);
                                                      }
                )
                                        .ExposeConfiguration(BuildSchema)
                                        .BuildSessionFactory();
        }

        private static void BuildSchema( Configuration configuration)
        {
            _configuration = configuration;

            configuration.SetProperty(Environment.ReleaseConnections, "on_close")
                .SetProperty(Environment.Dialect, typeof(SQLite.Test.SQLiteDialect).AssemblyQualifiedName)
                .SetProperty(Environment.ConnectionDriver, typeof (SQLite20Driver).AssemblyQualifiedName)
                //.SetProperty(Environment.ConnectionString, "data source=:memory:")
                .SetProperty("proxyfactory.factory_class", "NHibernate.ByteCode.LinFu.ProxyFactoryFactory, NHibernate.ByteCode.LinFu");

            //new SchemaExport(configuration).Create(false, true);
        }

        public ISession CreateSession()
        {
            ISession session = sessionFactory.OpenSession();
            new SchemaExport(_configuration).Execute(true, true, false, session.Connection, null);
            return session;
        }
    }
}